@@ -1,4 +1,4 @@
-(* $Id: mailbox.ml,v 1.2 2002/11/01 21:29:27 stolpmann Exp $
+(* $Id: mailbox.ml,v 1.3 2004/07/31 13:15:46 stolpmann Exp $
  * ----------------------------------------------------------------------
  *
  *)
@@ -66,10 +66,14 @@ let input_max ch s pos len =
   (* Try to read as much as possible *)
   let n = ref (ch # input s pos len) in
   let k = ref 1 in
-  while !n < len && !k <> 0 do
-    k := (ch # input s (pos + !n) (len - !n));
-    n := !n + !k
-  done;
+  ( try
+      while !n < len && !k <> 0 do
+	k := (ch # input s (pos + !n) (len - !n));
+	n := !n + !k
+      done;
+    with
+	End_of_file -> ()
+  );
   !n
 ;;
 
@@ -428,12 +432,12 @@ let restricted_html_attributes =
 ;;
 
 
-let http_re = Pcre.regexp "^\s*http:.*$" ;;
-let ftp_re = Pcre.regexp "^\s*ftp:.*$" ;;
-let mailto_re = Pcre.regexp "^\s*mailto:.*$" ;;
+let http_re = Pcre.regexp "^\\s*http:.*$" ;;
+let ftp_re = Pcre.regexp "^\\s*ftp:.*$" ;;
+let mailto_re = Pcre.regexp "^\\s*mailto:.*$" ;;
 let unrestricted_uris = [ http_re; ftp_re; mailto_re ] ;;
 
-let cid_uri_re = Pcre.regexp "^\s*cid:(.*)$";;
+let cid_uri_re = Pcre.regexp "^\\s*cid:(.*)$";;
     
 
 let clean_tree mk_cid_href html =
@@ -734,8 +738,11 @@ let display_part_fragment (cgi : cgi_activation) =
     List.map int_of_string 
       (Pcre.split ~rex:dot_re (cgi # argument_value "path")) in
 
-  let msg_hdr, `Body msg_body = 
-    extract_part (from_pos,header_pos,end_pos) path in
+  let msg_hdr, msg_body =
+    match extract_part (from_pos,header_pos,end_pos) path with
+	(hdr, `Body body) -> (hdr,body)
+      | _                 -> assert false
+  in
 
   if attachment then begin
     cgi # set_header
@@ -825,6 +832,9 @@ main();;
  * History:
  * 
  * $Log: mailbox.ml,v $
+ * Revision 1.3  2004/07/31 13:15:46  stolpmann
+ * 	Updated: End_of_file is caught in input_max
+ *
  * Revision 1.2  2002/11/01 21:29:27  stolpmann
  * 	Fix: The ~len argument of input_stream is now correct
  *
