@@ -1,4 +1,4 @@
-(* $Id: add_fastcgi.ml,v 1.2 2004/08/25 15:38:54 gremlin43820 Exp $
+(* $Id: add_fastcgi.ml,v 1.3 2004/11/24 18:01:07 gremlin43820 Exp $
  * ----------------------------------------------------------------------
  *
  *)
@@ -17,7 +17,7 @@ let text = Netencoding.Html.encode_from_latin1;;
  * as character entities. E.g. text "<" = "&lt;", and text "" = "&auml;"
  *)
 
-let begin_page cgi title =
+let begin_page (cgi: cgi_activation) title =
   (* Output the beginning of the page with the passed [title]. *)
   let out = cgi # output # output_string in
     out "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n";
@@ -25,6 +25,11 @@ let begin_page cgi title =
     (List.iter (* dump the header fields *)
        (fun (name, valu) -> out (name ^ ": " ^ valu ^ "\n"))
        cgi#environment#input_header_fields);
+    out "cookies\n--------\n";
+    (List.iter (* dump cookies *)
+       (fun (name, valu) -> out (name ^ ": " ^ valu ^ "\n"))
+       cgi#environment#cookies);
+    out ("remote host: " ^ (cgi#environment#cgi_remote_addr) ^ "\n");
     out "-->\n";
     out "<HTML>\n";
     out "<HEAD>\n";
@@ -165,8 +170,8 @@ serv process buffered_transactional_optype;;
  * History:
  * 
  * $Log: add_fastcgi.ml,v $
- * Revision 1.2  2004/08/25 15:38:54  gremlin43820
- * added a useful debugging feature. all the input header fields are printed in an html comment at the beginning of the page. I used this while debugging cookie propagation (or the lack of it) through the fastcgi connector, but it might be generally useful, so I will leave it.
+ * Revision 1.3  2004/11/24 18:01:07  gremlin43820
+ * commiting fastcgi compatibility patch from Kristof Pap which allows us to talk to a wider variety of fastcgi enabled web servers, and more closely follows the standard.
  *
  * Revision 1.1  2003/10/07 17:39:32  stolpmann
  * 	Imported Eric's patch for fastcgi
