@@ -1,4 +1,4 @@
-(* $Id: netcgi.ml,v 1.12 2004/07/01 12:56:20 stolpmann Exp $
+(* $Id: netcgi.ml,v 1.14 2005/02/02 21:19:47 gremlin43820 Exp $
  * ----------------------------------------------------------------------
  *
  *)
@@ -549,7 +549,9 @@ object (self)
     (* Check request method: *)
     let perm_methods = environ # config.permitted_http_methods in
     if not (List.mem (environ # cgi_request_method) perm_methods) then
-      failwith "Netcgi.std_activation: Request method is not permitted";
+      failwith ("Netcgi.std_activation: Request method: " ^ 
+		(environ # cgi_request_method) ^ 
+		" is not permitted");
 
     let content_length = try environ#input_content_length with Not_found -> 0 in
 
@@ -559,7 +561,8 @@ object (self)
       let perm_main_types = environ # config.permitted_input_content_types in
       if perm_main_types <> [] && 
 	not (List.mem content_main_type perm_main_types) then
-	  failwith "Netcgi.std_activation: Content type is not permitted";
+	  failwith ("Netcgi.std_activation: Content type: " ^ content_main_type ^
+		    " is not permitted");
     end;
 
     if content_length > 0 then
@@ -615,7 +618,7 @@ object (self)
       let decoder = Netmime.decode_mime_body arg_hdr in
       let disposition, disp_params = arg_hdr # content_disposition() in
       if disposition <> "form-data" then 
-	failwith "Netcgi.std_activation: Unknown Content-disposition";
+	failwith ("Netcgi.std_activation: Unknown Content-disposition: " ^ disposition);
       let name = 
 	try Mimestring.param_value(List.assoc "name" disp_params)
 	with Not_found -> failwith "Netcgi.std_activation: missing arg name" in
@@ -747,6 +750,9 @@ let tempfile_transactional_optype =
  * History:
  * 
  * $Log: netcgi.ml,v $
+ * Revision 1.14  2005/02/02 21:19:47  gremlin43820
+ * do the same for content type, and content disposition
+ *
  * Revision 1.12  2004/07/01 12:56:20  stolpmann
  * 	Removed dependency on Cgi.
  *
