@@ -1,4 +1,4 @@
-(* $Id: netcgi_env.ml,v 1.12 2002/10/24 23:47:48 stolpmann Exp $
+(* $Id: netcgi_env.ml,v 1.13 2004/11/24 18:01:13 gremlin43820 Exp $
  * ----------------------------------------------------------------------
  *
  *)
@@ -341,7 +341,7 @@ object (self)
     (* Note: ~soft_eol:"" because linear whitespace is illegal in CGI
      * responses
      *)
-    Mimestring.write_header ~soft_eol:"" ~eol:"\n" out_ch out_header#fields;
+    Mimestring.write_header ~soft_eol:"" ~eol:"\r\n" out_ch out_header#fields;
     out_state <- `Sent_header;
 
   method output_state = out_state
@@ -822,6 +822,9 @@ end ;;
  * History:
  * 
  * $Log: netcgi_env.ml,v $
+ * Revision 1.13  2004/11/24 18:01:13  gremlin43820
+ * commiting fastcgi compatibility patch from Kristof Pap which allows us to talk to a wider variety of fastcgi enabled web servers, and more closely follows the standard.
+ *
  * Revision 1.12  2002/10/24 23:47:48  stolpmann
  * 	Support for the HEAD method.
  * 	Workaround for a bug in MSIE: Empty cookies are represented
