@@ -1,4 +1,4 @@
-(* $Id: netcgi_fcgi.ml,v 1.6 2004/08/25 21:27:49 gremlin43820 Exp $
+(* $Id: netcgi_fcgi.ml,v 1.9 2005/02/03 19:16:53 gremlin43820 Exp $
  * ----------------------------------------------------------------------
  *
  *)
@@ -6,12 +6,13 @@
 (* This code is copyright 2003 Eric Stokes, and may be used under
 either, the GNU GPL, or the same license as ocamlnet *)
 
-open Netcgi_fcgi_10;;
-open Netcgi_env;;
-open Netcgi_types;;
-open Netcgi;;
-open Netchannels;;
-open Pcre;;
+open Netcgi_fcgi_10
+open Netcgi_env
+open Netcgi_types
+open Netcgi
+open Netchannels
+open Pcre
+open Unix
 
 (* classes which I must impliment
 class type raw_out_channel = object
@@ -85,7 +86,7 @@ let underscore = regexp ~study:true "_"
 let http = regexp ~study:true "^HTTP-"
 
 (* build a table of environment variables that we set explicitly. 
-   Fastcgi does not distinguish (much) between environment variables and http headers,
+   Fastcgi does not distinguish between environment variables and http headers,
    so when we set_input_header_fields (the method of passing on the http headers
    to the higher level) we need to filter out the environment variables. The reason
    we build a hashtbl outside the function is performance. We init the hash table at
@@ -155,15 +156,38 @@ let get_fcgi_activation optype =
 let serv (handler:(cgi_activation -> unit)) (optype:operating_type) =
   while true
   do
-    let cgi = get_fcgi_activation optype in
-      handler cgi;
-      cgi#output#close_out () (*close down this connection*)
+      let cgi = get_fcgi_activation optype in
+	try
+	  handler cgi;
+	  cgi#output#close_out () (*close down this connection*)
+	with FCGI_error (_, Unix_error (EPIPE, _, _)) -> ()
+	  (* clients who close the connection early will result in a
+	     sigpipe (which is ignored by default when using
+	     mod_fastcgi and returned via a Unix_error). This is
+	     normal behavior, and should NOT kill the application, 
+	     nor should each application writer be required to handle 
+	     it. We ensure two things with this handler.
+	     
+	     1. We will properly handle the sigpipe condition
+	     2. We will ONLY handle it if it comes from fastcgi,
+	        we should not mask bugs in the application. This is
+	        the greater part of the rational for creating the
+	        FCGI_error exception. *)
   done;;
 
 (* ======================================================================
  * History:
  * 
  * $Log: netcgi_fcgi.ml,v $
+ * Revision 1.9  2005/02/03 19:16:53  gremlin43820
+ * open unix in netcgi_fcgi.ml to fix compiler error. Make naming clearer in netcgi_fcgi_10.ml, in read_params, pluralize the list of params "param" is a confusing name for a list.
+ *
+ * Revision 1.8  2005/02/03 09:58:00  gremlin43820
+ * refine exception handling in low level fastcgi, and handle the case where the client closes the connection early gracefully.
+ *
+ * Revision 1.7  2005/02/02 23:57:36  gremlin43820
+ * fixed a bug in the new paramater reading functions. params with only a single paramater per packet would be completely omited. This resulted in getting no headers or environment variables at all on servers such as apache.
+ *
  * Revision 1.6  2004/08/25 21:27:49  gremlin43820
  * better handling of http header paramaters. Worked out what fastcgi is doing with the http headers vs the environment vars (they all come accross in the same structure), and added an algorythm for sorting them out. This fixes cookie support, which was broken (set_cookie worked, but you could not get cookies back), and probably some other obscure things which rely on http headers both standard and non-standard.
  *
