@@ -1,4 +1,4 @@
-(* $Id: mimestring.ml,v 2.15 2004/07/06 15:20:53 stolpmann Exp $
+(* $Id: mimestring.ml,v 2.16 2004/11/24 18:01:14 gremlin43820 Exp $
  * ----------------------------------------------------------------------
  *
  *)
@@ -133,10 +133,10 @@ let ws_re =
   S.regexp "^[ \t\r\n]*" ;;
 
 let write_header ?(soft_eol = "\r\n") ?(eol = "\r\n") ch h =
+  let hd = Buffer.create 120 in
   List.iter
     (fun (n,v) ->
-       ch # output_string n;
-       ch # output_string ": ";
+       Buffer.add_string hd (n ^ ": ");
        let l = String.length v in
        let pos = ref 0 in
        ( match S.string_match ws_re v 0 with
@@ -149,7 +149,7 @@ let write_header ?(soft_eol = "\r\n") ?(eol = "\r\n") ch h =
 	 while !pos < l do
 	   let (k, r) = S.search_forward write_re v !pos in  
 	                (* might raise Not_found *)
-	   ch # really_output v !pos (k - !pos);
+	   Buffer.add_substring hd v !pos (k - !pos);
 	   let pos' = S.match_end r in
 	   if pos' < l then begin 
 	     (* Before printing linear whitespace, ensure that the line
@@ -160,17 +160,17 @@ let write_header ?(soft_eol = "\r\n") ?(eol = "\r\n") ch h =
 		* spaces and TABs, but no CRs. Ensure that there is at least
 		* one space.
 		*)
-	       ch # output_string soft_eol;
+	       Buffer.add_string hd soft_eol;
 	       let found_space = ref false in
 	       for i = S.group_beginning r 2 to pos' - 1 do
 		 let c = v.[i] in
 		 if c = ' ' || c = '\t' then begin
-		   ch # output_char c;
+		   Buffer.add_char hd c;
 		   found_space := true;
 		 end
 	       done;
 	       if not !found_space then
-		 ch # output_char ' ';
+		 Buffer.add_char hd ' ';
 	     end
 	       (* else: The line would consist only of whitespace. We simply
 		* drop all of this whitespace.
@@ -178,13 +178,14 @@ let write_header ?(soft_eol = "\r\n") ?(eol = "\r\n") ch h =
 	   end;
 	   pos := pos'
 	 done;
-         ch # output_string eol;
+	 Buffer.add_string hd eol;
        with
 	   Not_found ->
 	     assert false  (* The reg exp matches always *)
     )
     h;
-  ch # output_string eol
+  Buffer.add_string hd eol;
+  ch # output_buffer hd
 ;;
 
 
@@ -1749,6 +1750,9 @@ let create_boundary ?(random = []) ?(nr = 0) () =
  * History:
  *
  * $Log: mimestring.ml,v $
+ * Revision 2.16  2004/11/24 18:01:14  gremlin43820
+ * commiting fastcgi compatibility patch from Kristof Pap which allows us to talk to a wider variety of fastcgi enabled web servers, and more closely follows the standard.
+ *
  * Revision 2.15  2004/07/06 15:20:53  stolpmann
  * 	Bugfixes: write_value now correctly folds long encoded words.
  *
