@@ -1,7 +1,9 @@
-(* $Id: netconversion.ml,v 2.15 2003/06/17 18:33:05 stolpmann Exp $
+(* $Id: netconversion.ml,v 2.20 2004/07/11 15:41:18 stolpmann Exp $
  * ----------------------------------------------------------------------
  *)
 
+open Netaux.ArrayAux
+
 exception Malformed_code
 
 exception Cannot_represent of int
@@ -84,11 +86,21 @@ type encoding =
   |  `Enc_cp874
   |  `Enc_cp1006
    (* IBM, EBCDIC-based: *)
-  |  `Enc_cp037
+  |  `Enc_cp037   (* EBCDIC USA Canada *)
+      (* 273: EBCDIC Germany, Austria,
+       * 277: Denmark, Norway,
+       * 278: Finland, Sweden,
+       * 280: Italy,
+       * 284: Spain, Latin America,
+       * 285: United Kingdom,
+       * 297: France,
+       * 871: Iceland,
+       *)
   |  `Enc_cp424
-  |  `Enc_cp500
-  |  `Enc_cp875
-  |  `Enc_cp1026
+  |  `Enc_cp500  (* EBCDIC International *)
+  |  `Enc_cp875  (* EBCDIC Modern Greek *)
+  |  `Enc_cp1026 (* EBCDIC Turkish *)
+  |  `Enc_cp1047 (* EBCDIC Latin1, OS 390 System Services *)
    (* Adobe: *)
   |  `Enc_adobe_standard_encoding
   |  `Enc_adobe_symbol_encoding
@@ -159,6 +171,7 @@ type charset =
   |  `Set_cp500
   |  `Set_cp875
   |  `Set_cp1026
+  |  `Set_cp1047
    (* Adobe: *)
   |  `Set_adobe_standard_encoding
   |  `Set_adobe_symbol_encoding
@@ -233,6 +246,8 @@ let names =
    * The aliases must not contain any of the punctuation characters
    * - _ .
    * `Enc_subset is missing in this list, of course.
+   *
+   * http://www.iana.org/assignments/character-sets
    *)
   [ `Enc_utf16,        [ "UTF-16"; "UTF16"; "UCS2"; "ISO10646UCS2" ];
     `Enc_utf16_be,     [ "UTF-16BE"; "UTF16BE" ];
@@ -243,16 +258,24 @@ let names =
 			 "ISOIR6" ];
     `Enc_iso88591,     [ "ISO-8859-1"; "ISO88591"; "LATIN1"; "CP819"; 
 			 "ISOIR100" ];
-    `Enc_iso88592,     [ "ISO-8859-2"; "ISO88592"; "LATIN2"; "ISOIR101" ];
+    `Enc_iso88592,     [ "ISO-8859-2"; "ISO88592"; "LATIN2"; "ISOIR101";
+			 "CP912"
+		       ];
     `Enc_iso88593,     [ "ISO-8859-3"; "ISO88593"; "LATIN3"; "ISOIR109" ];
     `Enc_iso88594,     [ "ISO-8859-4"; "ISO88594"; "LATIN4"; "ISOIR110" ];
-    `Enc_iso88595,     [ "ISO-8859-5"; "ISO88595"; "CYRILLIC"; "ISOIR144" ];
+    `Enc_iso88595,     [ "ISO-8859-5"; "ISO88595"; "CYRILLIC"; "ISOIR144";
+			 "CP915"
+		       ];
     `Enc_iso88596,     [ "ISO-8859-6"; "ISO88596"; "ARABIC"; "ECMA114"; 
-			 "ASMO708"; "ISOIR127" ];
+			 "ASMO708"; "ISOIR127"; "CP1089" ];
     `Enc_iso88597,     [ "ISO-8859-7"; "ISO88597"; "GREEK"; "GREEK8"; 
-			 "ELOT928"; "ECMA118"; "ISOIR126" ];
-    `Enc_iso88598,     [ "ISO-8859-8"; "ISO88598"; "HEBREW"; "ISOIR138" ];
-    `Enc_iso88599,     [ "ISO-8859-9"; "ISO88599"; "LATIN5"; "ISOIR148" ];
+			 "ELOT928"; "ECMA118"; "ISOIR126"; "CP813" ];
+    `Enc_iso88598,     [ "ISO-8859-8"; "ISO88598"; "HEBREW"; "ISOIR138";
+			 "CP916"
+		       ];
+    `Enc_iso88599,     [ "ISO-8859-9"; "ISO88599"; "LATIN5"; "ISOIR148";
+			 "CP920"
+		       ];
     `Enc_iso885910,    [ "ISO-8859-10"; "ISO885910"; "LATIN6"; "ISOIR157" ];
     `Enc_iso885911,    [ "ISO-8859-11"; "ISO885911"; ];
     `Enc_iso885913,    [ "ISO-8859-13"; "ISO885913"; "LATIN7" ];
@@ -273,30 +296,31 @@ let names =
     `Enc_windows1256,  [ "WINDOWS-1256"; "WINDOWS1256" ];
     `Enc_windows1257,  [ "WINDOWS-1257"; "WINDOWS1257" ];
     `Enc_windows1258,  [ "WINDOWS-1258"; "WINDOWS1258" ];
-    `Enc_cp437,        [ "CP437"; "CP437" ];
-    `Enc_cp737,        [ "CP737"; "CP737" ];
-    `Enc_cp775,        [ "CP775"; "CP775" ];
-    `Enc_cp850,        [ "CP850"; "CP850" ];
-    `Enc_cp852,        [ "CP852"; "CP852" ];
-    `Enc_cp855,        [ "CP855"; "CP855" ];
-    `Enc_cp856,        [ "CP856"; "CP856" ];
-    `Enc_cp857,        [ "CP857"; "CP857" ];
-    `Enc_cp860,        [ "CP860"; "CP860" ];
-    `Enc_cp861,        [ "CP861"; "CP861"; "CPIS" ];
-    `Enc_cp862,        [ "CP862"; "CP862" ];
-    `Enc_cp863,        [ "CP863"; "CP863" ];
-    `Enc_cp864,        [ "CP864"; "CP864" ];
-    `Enc_cp865,        [ "CP865"; "CP865" ];
-    `Enc_cp866,        [ "CP866"; "CP866" ];
-    `Enc_cp869,        [ "CP869"; "CP869"; "CPGR" ];
-    `Enc_cp874,        [ "CP874"; "CP874" ];
-    `Enc_cp1006,       [ "CP1006"; "CP1006" ];
-    `Enc_cp037,        [ "CP037"; "CP037"; "EBCDICCPUS"; "EBCDICCPCA"; 
+    `Enc_cp437,        [ "IBM437"; "CP437" ];
+    `Enc_cp737,        [ "IBM737"; "CP737" ];
+    `Enc_cp775,        [ "IBM775"; "CP775" ];
+    `Enc_cp850,        [ "IBM850"; "CP850" ];
+    `Enc_cp852,        [ "IBM852"; "CP852" ];
+    `Enc_cp855,        [ "IBM855"; "CP855" ];
+    `Enc_cp856,        [ "IBM856"; "CP856" ];
+    `Enc_cp857,        [ "IBM857"; "CP857" ];
+    `Enc_cp860,        [ "IBM860"; "CP860" ];
+    `Enc_cp861,        [ "IBM861"; "CP861"; "CPIS" ];
+    `Enc_cp862,        [ "IBM862"; "CP862" ];
+    `Enc_cp863,        [ "IBM863"; "CP863" ];
+    `Enc_cp864,        [ "IBM864"; "CP864" ];
+    `Enc_cp865,        [ "IBM865"; "CP865" ];
+    `Enc_cp866,        [ "IBM866"; "CP866" ];
+    `Enc_cp869,        [ "IBM869"; "CP869"; "CPGR" ];
+    `Enc_cp874,        [ "IBM874"; "CP874" ];
+    `Enc_cp1006,       [ "IBM1006"; "CP1006" ];
+    `Enc_cp037,        [ "IBM037"; "CP037"; "EBCDICCPUS"; "EBCDICCPCA"; 
 			 "EBCDICCPWT"; "EBCDICCPNL" ];
-    `Enc_cp424,        [ "CP424"; "CP424"; "EBCDICCPHE" ];
-    `Enc_cp500,        [ "CP500"; "CP500"; "EBCDICCPBE"; "EBCDICCPCH" ];
-    `Enc_cp875,        [ "CP875"; "CP875" ];
-    `Enc_cp1026,       [ "CP1026"; "CP1026" ];
+    `Enc_cp424,        [ "IBM424"; "CP424"; "EBCDICCPHE" ];
+    `Enc_cp500,        [ "IBM500"; "CP500"; "EBCDICCPBE"; "EBCDICCPCH" ];
+    `Enc_cp875,        [ "IBM875"; "CP875" ];
+    `Enc_cp1026,       [ "IBM1026"; "CP1026" ];
+    `Enc_cp1047,       [ "IBM1047"; "CP1047"; ];
     `Enc_adobe_standard_encoding,       [ "ADOBE-STANDARD-ENCODING"; 
 					  "ADOBESTANDARDENCODING" ];
     `Enc_adobe_symbol_encoding,         [ "ADOBE-SYMBOL-ENCODING"; 
@@ -394,6 +418,7 @@ let internal_name (cs : charset) =
     | `Set_cp500 -> "cp500"
     | `Set_cp875 -> "cp875"
     | `Set_cp1026 -> "cp1026"
+    | `Set_cp1047 -> "cp1047"
     | `Set_adobe_standard_encoding -> "adobe_standard_encoding"
     | `Set_adobe_symbol_encoding -> "adobe_symbol_encoding"
     | `Set_adobe_zapf_dingbats_encoding -> "adobe_zapf_dingbats_encoding"
@@ -460,6 +485,7 @@ let rec required_charsets (e : encoding) =
     | `Enc_cp500 -> [ `Set_cp500 ]
     | `Enc_cp875 -> [ `Set_cp875 ]
     | `Enc_cp1026 -> [ `Set_cp1026 ]
+    | `Enc_cp1047 -> [ `Set_cp1047 ]
     | `Enc_adobe_standard_encoding -> [ `Set_adobe_standard_encoding ]
     | `Enc_adobe_symbol_encoding -> [ `Set_adobe_symbol_encoding ]
     | `Enc_adobe_zapf_dingbats_encoding -> [ `Set_adobe_zapf_dingbats_encoding ]
@@ -581,9 +607,12 @@ let available_output_encodings() =
  *  - The validity of the input encoding needs not to be checked
  *)
 
-exception Malformed_code_read of (int * int * encoding)
+exception Malformed_code_read of (int * int * encoding);;
   (* not exported! *)
 
+Callback.register_exception "Netconversion.Malformed_code_read"
+			    (Malformed_code_read(0,0,`Enc_empty));;
+  (* Needed by netaccel_c.c *)
 
 
 
@@ -618,6 +647,9 @@ let read_iso88591 maxcode enc slice_char slice_blen s_in p_in l_in =
 ;;
 
 
+let read_iso88591_ref = ref read_iso88591;;
+
+
 let get_8bit_to_unicode_map enc =
   let cs = 
     match required_charsets enc with
@@ -816,11 +848,15 @@ let read_utf8 is_java slice_char slice_blen s_in p_in l_in =
 ;;
 
 
+let read_utf8_ref = ref read_utf8;;
+
+
 let surrogate_offset = 0x10000 - (0xD800 lsl 10) - 0xDC00;;
 
-let read_utf16_lebe lo hi enc slice_char slice_blen s_in p_in l_in =
+let read_utf16_lebe lo hi n_start enc slice_char slice_blen s_in p_in l_in =
   (* lo=0, hi=1: little endian
    * lo=1, hi=0: big endian
+   * n_start: First cell in slice to use
    *)
   assert(Array.length slice_char = Array.length slice_blen);
   assert(p_in >= 0 && p_in + l_in <= String.length s_in && l_in >= 0);
@@ -872,7 +908,7 @@ let read_utf16_lebe lo hi enc slice_char slice_blen s_in p_in l_in =
     else
       (n,k)
   in
-  let (n,k) = put_loop 0 0 in
+  let (n,k) = put_loop 0 n_start in
   if n < Array.length slice_char then (
     (* EOF marker *)
     slice_char.(n) <- (-1);
@@ -881,36 +917,58 @@ let read_utf16_lebe lo hi enc slice_char slice_blen s_in p_in l_in =
 ;;
 
 
-let read_utf16 slice_char slice_blen s_in p_in l_in =
+let get_endianess s_in p_in =
+  let c0 = s_in.[p_in + 0] in
+  let c1 = s_in.[p_in + 1] in
+  if c0 = '\254' && c1 = '\255' then
+    `Big_endian
+  else
+    if c0 = '\255' && c1 = '\254' then
+      `Little_endian
+    else
+      `No_BOM
+;;
+
+
+(* expose_bom: when true, the BOM is considered as a character and
+ * put as value (-3) into slice_char
+ *)
+
+let read_utf16 expose_bom slice_char slice_blen s_in p_in l_in =
   assert(Array.length slice_char = Array.length slice_blen);
   assert(p_in >= 0 && p_in + l_in <= String.length s_in && l_in >= 0);
   (* Expect a BOM at the beginning of the text *)
   if l_in >= 2 then begin
-    let c0 = s_in.[p_in + 0] in
-    let c1 = s_in.[p_in + 1] in
-    if c0 = '\254' & c1 = '\255' then begin
-      (* 0xfeff as big endian *)
-      let (n, k, enc') = 
-	read_utf16_lebe 1 0 `Enc_utf16_be slice_char slice_blen s_in (p_in+2) (l_in-2) in
-      if n > 0 then slice_blen.(0) <- slice_blen.(0) + 2;
-      (n, k+2, enc')
-    end
-    else 
-      if c0 = '\255' & c1 = '\254' then begin
-	(* 0xfeff as little endian *)
-	let (n, k, enc') = 
-	  read_utf16_lebe 0 1 `Enc_utf16_le slice_char slice_blen s_in (p_in+2) (l_in-2) in
-	if n > 0 then slice_blen.(0) <- slice_blen.(0) + 2;
-	(n, k+2, enc')
-      end
-      else (
-	(* byte order mark missing *)
-	slice_char.(0) <- (-1);
-	raise(Malformed_code_read(0,0,`Enc_utf16))
-      )
+    if expose_bom then (
+      slice_char.(0) <- (-3); 
+      slice_blen.(0) <- 0;  (* Later corrected *)
+    );
+    match get_endianess s_in p_in with
+	`Big_endian ->
+	  let n_start = if expose_bom then 1 else 0 in
+	  let (n, k, enc') = 
+	    read_utf16_lebe
+	      1 0 n_start `Enc_utf16_be 
+	      slice_char slice_blen s_in (p_in+2) (l_in-2) in
+	  if n > 0 then slice_blen.(0) <- slice_blen.(0) + 2;
+	  (n, k+2, enc')
+      | `Little_endian ->
+	  let n_start = if expose_bom then 1 else 0 in
+	  let (n, k, enc') = 
+	    read_utf16_lebe 
+	      0 1 n_start `Enc_utf16_le 
+	      slice_char slice_blen s_in (p_in+2) (l_in-2) in
+	  if n > 0 then slice_blen.(0) <- slice_blen.(0) + 2;
+	  (n, k+2, enc')
+      | `No_BOM ->
+	  (* byte order mark missing *)
+	  slice_char.(0) <- (-1);
+	  raise(Malformed_code_read(0,0,`Enc_utf16))
   end
-  else
+  else (
+    slice_char.(0) <- (-1);
     (0, 0, `Enc_utf16)
+  )
 ;;
 
 
@@ -1068,8 +1126,8 @@ let read_subset inner_read def slice_char slice_blen s_in p_in l_in =
 	    Malformed_code_read(_,_,_) -> assert false
       in
       assert(n' = j);
-      Array.blit slice_char' 0 slice_char 0 j;
-      Array.blit slice_blen' 0 slice_blen 0 j;
+      int_blit slice_char' 0 slice_char 0 j;
+      int_blit slice_blen' 0 slice_blen 0 j;
       slice_char.(j) <- (-1);
       raise (Malformed_code_read(j, k', enc''))
     );
@@ -1770,25 +1828,41 @@ let rec from_unicode cs =
 ;;
 
 
-let rec get_reader enc =
+type encoding1 =
+  [ encoding | `Enc_utf16_bom ] ;;
+
+  (* `Enc_utf16_bom considers the BOM as a character with code point -3.
+   * This encoding is only internally used.
+   *)
+
+let rec get_reader1 (enc : encoding1) =
+  (* get_reader1 supports the additional internal encodings of
+   * encoding1. get_reader (below) only supports the exported
+   * encodings.
+   *)
   match enc with
-      `Enc_iso88591 -> read_iso88591 255 enc
-    | `Enc_usascii  -> read_iso88591 127 enc
-    | `Enc_empty    -> read_iso88591 (-1) enc
-    | `Enc_utf8     -> read_utf8 false
-    | `Enc_java     -> read_utf8 true
-    | `Enc_utf16    -> read_utf16
-    | `Enc_utf16_le -> read_utf16_lebe 0 1 enc
-    | `Enc_utf16_be -> read_utf16_lebe 1 0 enc
+      `Enc_iso88591 -> !read_iso88591_ref 255 `Enc_iso88591
+    | `Enc_usascii  -> !read_iso88591_ref 127 `Enc_usascii
+    | `Enc_empty    -> !read_iso88591_ref (-1) `Enc_empty
+    | `Enc_utf8     -> !read_utf8_ref false
+    | `Enc_java     -> !read_utf8_ref true
+    | `Enc_utf16    -> read_utf16 false
+    | `Enc_utf16_bom  -> read_utf16 true
+    | `Enc_utf16_le -> read_utf16_lebe 0 1 0 `Enc_utf16_le
+    | `Enc_utf16_be -> read_utf16_lebe 1 0 0 `Enc_utf16_be
     | `Enc_eucjp    -> read_eucjp ()
     | `Enc_subset(e,def) ->
-	let reader' = get_reader e in
+	let reader' = get_reader1 (e :> encoding1) in
 	read_subset reader' def
-    | _ -> 
-	read_8bit enc
+    | #encoding as e -> 
+	read_8bit (e :> encoding)
 ;;
 
 
+let get_reader = 
+  (get_reader1 : encoding1 -> 'a :> encoding -> 'a);;
+
+
 let rec get_writer enc =
   match enc with
       `Enc_iso88591 -> write_iso88591 255
@@ -2089,26 +2163,59 @@ class recoding_pipe ?(subst = (fun p -> raise Not_found))
  * the cursor forward. The slice array stores the scanned characters.
  * If the read call raises Malformed_code, the size of the slice
  * is decreased to 1, so the exact position can be calculated.
+ *)
+
+(* Notes UTF-16:
+ * 
+ * cursor_enc is updated after the first slice has been read. This
+ * usually changes this field to either the big or little endian
+ * encoding variant. No update is needed when previous slices are
+ * scanned, because the BOM is only allowed at the beginning of the
+ * string, so can at most go back to exactly the BOM.
  *
- * TODO: manage mutable encodings
+ * cursor_encoding returns `Enc_utf16 when the cursor is over the
+ * BOM, and cursor_enc otherwise.
  *)
 
 exception End_of_string;;
 exception Cursor_out_of_range;;
+exception Partial_character;;
+exception Byte_order_mark;;
 
 
 type cursor =
     { (* configuration: *)
-      cursor_target : string;
-      cursor_range_pos : int;
-      cursor_range_len : int;
-      cursor_offset : int;
+      mutable cursor_target : string;
+      mutable cursor_range_pos : int;
+      mutable cursor_range_len : int;
+      mutable cursor_offset : int;
       mutable cursor_enc : encoding;
+        (* `Enc_utf16: Only used if the slice or string is too short to
+         *    recognize the endianess. Otherwise, the encoding is set
+         *    to the endian-aware variant.
+         *)
+      mutable cursor_has_bom : bool;
+        (* Whether there is a BOM. Only when initially cursor_enc=`Enc_utf16 *)
       (* conversion: *)
       mutable cursor_slice_char : int array;
-        (* Contains the characters of the slice, -1 is EOF *)
+        (* Contains the characters of the slice. Special values:
+	 * -1: EOF
+	 * -2: Incomplete multi-byte character at end
+	 * -3: BOM at the beginning
+	 *)
       mutable cursor_slice_blen : int array;
-        (* Contains the byte length of the characters *)
+        (* Contains the byte length of the characters.
+	 * Recall that this array must contain 1s when single-byte
+	 * encodings are scanned, and must not be modified. The
+	 * "reader" does not fill this array for single-byte encodings,
+	 * so modifications would persist!
+	 *)
+      mutable cursor_imbchar_len : int;
+        (* This is always 0, except in the special case when the first
+	 * character of this slice is EOF, and EOF is preceded by an
+	 * incomplete multi-byte character (imbchar). In this case,
+	 * the length of the imbchar is stored here.
+	 *)
       mutable cursor_slice_char_pos : int;
         (* char pos of the beginning of the slice *)
       mutable cursor_slice_byte_pos : int;
@@ -2158,12 +2265,31 @@ let cursor_char_count cs = cs.cursor_char_pos;;
 
 let cursor_pos cs = cs.cursor_byte_pos;;
 
-let cursor_encoding cs = cs.cursor_enc;;
+let cursor_encoding cs = 
+  let enc = cs.cursor_enc in
+  match enc with
+    ( `Enc_utf16_le
+    | `Enc_utf16_be ) when cs.cursor_has_bom ->
+	if cs.cursor_byte_pos = cs.cursor_range_pos then
+	  `Enc_utf16
+	else
+	  enc
+    | _ ->
+	enc
+;;
+
+
+exception Failing_in_Netconversion_uchar_at
 
 let uchar_at cs =
   let ch = cs.cursor_slice_char.(cs.cursor_index) in
-  if ch = -1 then
-    raise End_of_string
+  if ch < 0 then
+    match ch with
+	-1 -> raise End_of_string
+      | -2 -> raise Partial_character
+      | -3 -> raise Byte_order_mark
+      | _  -> (* assert false *) raise Failing_in_Netconversion_uchar_at
+	      (* "assert false" isn't inlined! *)
   else
     ch
 ;;
@@ -2282,29 +2408,81 @@ let move ?(num = 1) cs =
 
 let init_load_slice cs enc =
 
-  let reader = get_reader enc in
-  let back   = get_back_fn enc in
+  let reader0 = get_reader enc in
+  let back0   = lazy(get_back_fn enc) in
+  (* For most encodings, [reader] and [back] never change.
+   * For UTF-16, there may be refinements, however.
+   *)
+
+  let reader() =
+    match cs.cursor_enc with
+      ( `Enc_utf16
+      | `Enc_utf16_le
+      | `Enc_utf16_be ) when cs.cursor_has_bom ->
+	  (* Ensure we use `Enc_utf16_bom when we read the beginning
+	   * of the range
+	   *)
+	  (fun slice_char slice_blen s bp bl ->
+	     if bp = cs.cursor_range_pos then
+	       get_reader1 `Enc_utf16_bom slice_char slice_blen s bp bl
+	     else
+	       get_reader cs.cursor_enc slice_char slice_blen s bp bl
+	  )
+      | _ ->
+	  reader0
+  in
+
+  let back() =
+    match cs.cursor_enc with
+      ( `Enc_utf16
+      | `Enc_utf16_le
+      | `Enc_utf16_be ) when cs.cursor_has_bom ->
+	  get_back_fn cs.cursor_enc
+      | _ ->
+	  Lazy.force back0
+  in
+
+  let record_imbchar rd_chars rd_bytes scan_bytes =
+    (* Put a (-2) at position rd_chars of the current slice *)
+    cs.cursor_slice_char.(rd_chars) <- (-2);
+    cs.cursor_slice_blen.(rd_chars) <- scan_bytes - rd_bytes;
+    if rd_chars+1 < Array.length cs.cursor_slice_char then (
+      cs.cursor_slice_char.(rd_chars+1) <- (-1);
+    );
+    cs.cursor_slice_length <- rd_chars+1;
+    cs.cursor_slice_bytes <- scan_bytes;
+  in
 
   let repair_slice slice_size =
     (* When the reader raises Malformed_code, the slice has been
      * modified. This function reloads the old slice again.
+     *
+     * This function repairs these fields from the other fields:
+     * - cursor_slice_char
+     * - cursor_slice_blen
+     * - cursor_slice_length
+     * - cursor_slice_bytes
      *)
+    let bp = cs.cursor_slice_byte_pos in
     let ep = cs.cursor_range_pos + cs.cursor_range_len in
     let (slice_char, slice_blen) =
       (Array.create slice_size (-1), Array.create slice_size 1) in
     let rd_chars, rd_bytes, _ =
       try
 	reader 
-          slice_char slice_blen cs.cursor_target 
-	  cs.cursor_slice_byte_pos (ep-cs.cursor_slice_byte_pos) 
+          () slice_char slice_blen cs.cursor_target bp (ep-bp) 
       with
 	  (* should not happen: *)
 	  Malformed_code_read(_,_,_) -> raise Malformed_code
     in
-    assert(cs.cursor_slice_length = rd_chars);
-    assert(cs.cursor_slice_bytes = rd_bytes);
+    cs.cursor_slice_length <- rd_chars;
+    cs.cursor_slice_bytes <- rd_bytes;
     cs.cursor_slice_char <- slice_char;
-    cs.cursor_slice_blen <- slice_blen
+    cs.cursor_slice_blen <- slice_blen;
+    (* Check for imbchars: *)
+    if rd_chars < slice_size && rd_bytes < ep-bp then (
+      record_imbchar rd_chars rd_bytes (ep-bp);
+    );
   in
 
   let load_next_slice() =
@@ -2312,6 +2490,16 @@ let init_load_slice cs enc =
     let bp = cs.cursor_byte_pos in
     let ep = cs.cursor_range_pos + cs.cursor_range_len in
     let load slice_size =
+      let old_partial_len =
+	(* Handle the case that the last character is (-2), and thus the first
+	 * character of the next slice will be (-1). Then, cursor_imbchar_len
+	 * must be set to the length of the (-2) character.
+	 *)
+	if cs.cursor_slice_char.(cs.cursor_slice_length-1) = (-2) then
+	  cs.cursor_slice_blen.(cs.cursor_slice_length-1)
+	else
+	  0
+      in
       (* Check if the current array can be reused, or if we need new
        * arrays with different sizes
        *)
@@ -2326,14 +2514,9 @@ let init_load_slice cs enc =
       (* Use the reader to decode the bytes and to put the characters into
        * slice_char. 
        *)
-      let rd_chars, rd_bytes, _ =
-	reader slice_char slice_blen cs.cursor_target bp (ep-bp) in
+      let rd_chars, rd_bytes, enc' =
+	reader () slice_char slice_blen cs.cursor_target bp (ep-bp) in
         (* may raise Malformed_code_read *)
-      (* If the slice is incomplete, we are at the end of the range.
-       * Check whether there are multi-byte prefixes 
-       *)
-      if rd_chars < slice_size && rd_bytes < ep-bp then
-	raise Malformed_code;
       (* Update cursor record: *)
       cs.cursor_index <- 0;
       cs.cursor_slice_char <- slice_char;
@@ -2342,6 +2525,12 @@ let init_load_slice cs enc =
       cs.cursor_slice_bytes <- rd_bytes;
       cs.cursor_slice_char_pos <- cp;
       cs.cursor_slice_byte_pos <- bp;
+      cs.cursor_imbchar_len <- old_partial_len;
+      cs.cursor_enc <- enc';
+      (* Check for imbchars: *)
+      if rd_chars < slice_size && rd_bytes < ep-bp then (
+	record_imbchar rd_chars rd_bytes (ep-bp);
+      );
     in
     (* Is the cursor positioned in a problem range? If yes, decode only
      * one character. If not, try to decode a block of characters.
@@ -2389,35 +2578,48 @@ let init_load_slice cs enc =
 	  (Array.create slice_size (-1), Array.create slice_size 1)
 	  (* create new arrays with different size *)
       in
-      (* Go back 1 character: *)
+      (* Go back 1 character (must always succeed): *)
       if bp = cs.cursor_range_pos then raise Cursor_out_of_range;      
       let bk1_chars, bk1_bytes =
-	back 
-	  cs.cursor_target cs.cursor_range_pos bp 1 in
+	if cs.cursor_imbchar_len > 0 then
+	  (* Special case: the last character of this slice is an imbchar.
+	   * Assume imbchar_len 
+	   *)
+	  (1, cs.cursor_imbchar_len)
+	else
+	  back 
+	    () cs.cursor_target cs.cursor_range_pos bp 1 in
       if bk1_chars = 0 then raise Malformed_code;
       (* bk1_chars = 0: this means there is a multi-byte suffix at the
        * beginning of the range
+       * ==> bk1_chars = 1
        *)
-      (* Go back slice_size characters: *)
+      (* Go back further slice_size-1 characters (or less): *)
       let bk_chars, bk_bytes =
 	back 
-	  cs.cursor_target cs.cursor_range_pos bp slice_size in
-      let bp' = bp - bk_bytes in
+	  () cs.cursor_target cs.cursor_range_pos (bp-bk1_bytes) (slice_size-1) in
+      let bp' = bp - bk1_bytes - bk_bytes in
       (* Use the reader to decode the bytes and to put the characters into
        * slice_char. 
        *)
       let rd_chars, rd_bytes, _ =
-	reader slice_char slice_blen cs.cursor_target bp' (ep-bp') in
+	reader () slice_char slice_blen cs.cursor_target bp' (ep-bp') in
         (* may raise Malformed_code_read *)
       assert(rd_chars >= bk_chars);
       (* Update cursor record: *)
-      cs.cursor_index <- bk_chars - 1;
+      cs.cursor_index <- bk_chars;
       cs.cursor_slice_char <- slice_char;
       cs.cursor_slice_blen <- slice_blen;
       cs.cursor_slice_length <- rd_chars;
       cs.cursor_slice_bytes <- rd_bytes;
-      cs.cursor_slice_char_pos <- cp - bk_chars;
+      cs.cursor_slice_char_pos <- cp - bk_chars - 1;
       cs.cursor_slice_byte_pos <- bp';
+      cs.cursor_imbchar_len <- 0; (* Cannot happen *)
+      (* Don't need to update cursor_enc! *)
+      (* Check for imbchars: *)
+      if rd_chars < slice_size && rd_bytes < ep-bp' then (
+	record_imbchar rd_chars rd_bytes (ep-bp');
+      );
       (* Implicitly go one character back: *)
       cs.cursor_char_pos <- cp - 1;
       cs.cursor_byte_pos <- bp - bk1_bytes;
@@ -2477,8 +2679,8 @@ let create_cursor ?(range_pos = 0) ?range_len ?(initial_rel_pos = 0) enc s =
   if initial_rel_pos < 0 || initial_rel_pos > range_len then
     invalid_arg "Netconversion.create_cursor";
 
-  if enc = `Enc_utf16 then
-    failwith "Netconversion.create_cursor: The encoding `Enc_utf16 is not supported";
+  if enc = `Enc_utf16 && initial_rel_pos <> 0 then
+    failwith "Netconversion.create_cursor: The encoding `Enc_utf16 only supported when initial_rel_pos=0";
 
   let cs =
     { cursor_target = s;
@@ -2486,8 +2688,10 @@ let create_cursor ?(range_pos = 0) ?range_len ?(initial_rel_pos = 0) enc s =
       cursor_range_len = range_len;
       cursor_offset = initial_rel_pos;
       cursor_enc = enc;
+      cursor_has_bom = (enc = `Enc_utf16);
       cursor_slice_char = [| 1 |];
       cursor_slice_blen = [| 1 |];
+      cursor_imbchar_len = 0;
       cursor_slice_char_pos = 0;
       cursor_slice_byte_pos = range_pos + initial_rel_pos;
       cursor_slice_length = 1;
@@ -2510,6 +2714,65 @@ let create_cursor ?(range_pos = 0) ?range_len ?(initial_rel_pos = 0) enc s =
 ;;
 
 
+let reinit_cursor ?(range_pos = 0) ?range_len ?(initial_rel_pos = 0) ?enc s cs =
+  if range_pos < 0 || range_pos > String.length s then
+    invalid_arg "Netconversion.reinit_cursor";
+  
+  let range_len =
+    match range_len with
+	Some l -> l
+      | None   -> String.length s - range_pos in
+  if range_len < 0 || range_pos + range_len > String.length s then
+    invalid_arg "Netconversion.reinit_cursor";
+
+  if initial_rel_pos < 0 || initial_rel_pos > range_len then
+    invalid_arg "Netconversion.reinit_cursor";
+
+  let enc = 
+    match enc with
+	None -> cs.cursor_enc
+      | Some e -> e
+  in
+  if enc = `Enc_utf16 && initial_rel_pos <> 0 then
+    failwith "Netconversion.reinit_cursor: The encoding `Enc_utf16 only supported when initial_rel_pos=0";
+
+  let old_enc = cs.cursor_enc in
+
+  cs.cursor_target <- s;
+  cs.cursor_range_pos <- range_pos;
+  cs.cursor_range_len <- range_len;
+  cs.cursor_offset <- initial_rel_pos;
+  cs.cursor_enc <- enc;
+  cs.cursor_has_bom <- (enc = `Enc_utf16);
+  cs.cursor_imbchar_len <- 0;
+  cs.cursor_slice_char_pos <- 0;
+  cs.cursor_slice_byte_pos <- range_pos + initial_rel_pos;
+  cs.cursor_slice_length <- 1;
+  cs.cursor_slice_bytes <- 0;
+  cs.cursor_char_pos <- 0;
+  cs.cursor_byte_pos <- range_pos + initial_rel_pos;
+  cs.cursor_index <- 1;
+  cs.cursor_problem_range_start <- max_int;
+  cs.cursor_problem_range_end <- max_int;
+  cs.load_next_slice <- (fun () -> assert false);
+  cs.load_prev_slice <- (fun () -> assert false);
+  
+  cs.cursor_slice_char.(0) <- 1;
+  if not (same_encoding enc old_enc) then
+    (* slice_blen: It might have happened that the new encoding is an
+     * 8 bit charset, but the old was not. Re-initialize this array
+     * to ensure it contains only "1" in this case.
+     *)
+    Array.fill cs.cursor_slice_blen 0 (Array.length cs.cursor_slice_blen) 1;
+
+  init_load_slice cs enc;
+  
+  (* load the next slice to do the rest of the initialization: *)
+  cs.load_next_slice();
+
+;;
+
+
 let copy_cursor ?enc cs =
   let enc = 
     match enc with
@@ -2544,10 +2807,58 @@ let copy_cursor ?enc cs =
 ;;
 
 
+let cursor_blit_maxlen cs = 
+  let l = cs.cursor_slice_length - cs.cursor_index in
+  (* Test on special situations: *)
+  match cs.cursor_slice_char.(cs.cursor_index) with
+      -1 -> (* EOF *) raise End_of_string
+    | -3 -> (* BOM *) 0
+    | _ ->
+	if cs.cursor_slice_char.(cs.cursor_slice_length - 1) = (-2) then
+	  (* Partial character *)
+	  l-1
+	else
+	  l
+;;
+
+
+let cursor_blit cs ua pos len =
+  if pos < 0 || len < 0 || pos+len > Array.length ua then
+    invalid_arg "Netconversion.cursor_blit";
+  let cs_len = cursor_blit_maxlen cs in
+  let l = min cs_len len in
+  int_blit cs.cursor_slice_char cs.cursor_index ua pos l;
+  l
+;;
+
+
+let cursor_blit_positions cs ua pos len =
+  if pos < 0 || len < 0 || pos+len > Array.length ua then
+    invalid_arg "Netconversion.cursor_blit_positions";
+  let cs_len = cursor_blit_maxlen cs in
+  let l = min cs_len len in
+  let p = cs.cursor_byte_pos in
+  let blen = cs.cursor_slice_blen in
+  let cidx = cs.cursor_index in
+  assert(pos+l <= Array.length ua);
+  assert(cidx+l <= Array.length cs.cursor_slice_blen);
+  Netaux.ArrayAux.int_series blen cidx ua pos l p;
+  l
+;;
+
+
 (**********************************************************************)
 (* String functions                                                   *)
 (**********************************************************************)
 
+(* CHECK
+ * - ustring_length: Count imbchars? No!
+ *
+ * DOC:
+ * - imbchar handling (additional exceptions)
+ *)
+
+
 let ustring_length enc =
   if is_single_byte enc then
     fun ?(range_pos=0) ?range_len s ->
@@ -2565,7 +2876,16 @@ let ustring_length enc =
        *)
       let cs = create_cursor ?range_pos ?range_len enc s in
       ( try move ~num:max_int cs with Cursor_out_of_range -> ());
-      cursor_char_count cs
+      let n = cursor_char_count cs in
+      (* Check that the last char is not an imbchar *)
+      ( try
+	  move ~num:(-1) cs;
+	  let _ = uchar_at cs in ()
+	with
+	    Cursor_out_of_range -> ()
+	  | Partial_character -> raise Malformed_code
+      );
+      n
 ;;
 
 
@@ -2588,6 +2908,24 @@ let verify enc ?range_pos ?range_len s =
 	  let n = cs.cursor_slice_blen.(cs.cursor_index) in
 	  raise (Malformed_code_at (cs.cursor_byte_pos + n))
   );
+  (* Now we are at EOF. Check this. Furthermore, check whether there is
+   * an imbchar just before EOF:
+   *)
+  ( try
+      let _ = uchar_at cs in
+      assert false
+    with
+	End_of_string -> ()
+  );
+  ( try
+      move ~num:(-1) cs;
+      let _ = uchar_at cs in
+      ()
+    with
+	Cursor_out_of_range -> ()  (* empty string *)
+      | Partial_character ->
+	  raise (Malformed_code_at (cs.cursor_byte_pos))
+  );
   ()
 ;;
   
@@ -2605,16 +2943,32 @@ let ustring_iter enc f ?range_pos ?range_len s =
   with
       End_of_string ->
 	()
+    | Partial_character ->
+	raise Malformed_code
 ;;
 
 
-let ustring_map enc f =
-  let mkch = ustring_of_uchar enc in
-  let subst p =
-    let p' = f p in
-    String.concat "" (List.map mkch p')
-  in
-  convert ~subst ~in_enc:enc ~out_enc:`Enc_empty
+let ustring_map enc f ?range_pos ?range_len s =
+  (* The following algorithm works only if the mapped lists are short:
+     let mkch = ustring_of_uchar enc in
+     let subst p =
+     let p' = f p in
+       String.concat "" (List.map mkch p')
+     in
+     convert ~subst ~in_enc:enc ~out_enc:`Enc_empty ?range_pos ?range_len s
+  *)
+  let buf = Buffer.create 250 in
+  ustring_iter
+    enc
+    (fun p ->
+       let l = f p in
+       Buffer.add_string 
+	 buf (String.concat "" (List.map (ustring_of_uchar enc) l))
+    )
+    ?range_pos
+    ?range_len
+    s;
+  Buffer.contents buf
 ;;
 
 
@@ -2626,9 +2980,15 @@ let ustring_sub enc pos len ?range_pos ?range_len s =
     let byte_pos_0 = cursor_pos cs in
     move ~num:len cs;
     let byte_pos_1 = cursor_pos cs in
+    (* Check: The last character of the string must not be an imbchar *)
+    if len > 0 then (
+      move ~num:(-1) cs;
+      let _ = uchar_at cs in ();  (* or Partial_character *)
+    );
     String.sub s byte_pos_0 (byte_pos_1 - byte_pos_0)
   with
       Cursor_out_of_range -> invalid_arg "Netconversion.ustring_sub"
+    | Partial_character -> raise Malformed_code
 ;;
 
 
@@ -2652,6 +3012,7 @@ let ustring_compare enc f ?range_pos:rp1 ?range_len:rl1 s1
 	    | true, true -> 0
 	    | _ -> assert false
 	)
+    | Partial_character -> raise Malformed_code
 ;;
 
 
@@ -2752,6 +3113,25 @@ let ustring_of_uarray ?(subst = fun code -> raise (Cannot_represent code))
  * History:
  * 
  * $Log: netconversion.ml,v $
+ * Revision 2.20  2004/07/11 15:41:18  stolpmann
+ * 	Added: IBM-1047 charset
+ *
+ * Revision 2.19  2004/07/08 22:10:18  stolpmann
+ * 	Adding accelerators for character set conversion.
+ *
+ * Revision 2.18  2004/07/07 22:41:53  stolpmann
+ * 	Added Netulex, Netaux.ArrayAux.
+ * 	Some improvements in Netconversion.
+ *
+ * Revision 2.17  2004/06/20 13:12:37  stolpmann
+ * 	Added some encoding names (code pages)
+ *
+ * Revision 2.16  2004/06/09 22:51:56  stolpmann
+ * 	Enhanced cursors: Support for UTF-16 with BOMs, support for
+ * incomplete multi-byte characters,b litting functions.
+ * 	Added ocamldoc to Netconversion.
+ * 	Updated test_recode: New tests for cursors.
+ *
  * Revision 2.15  2003/06/17 18:33:05  stolpmann
  * 	Fix: "recode" works now if the reader runs into an invalid
  * byte sequence that should be ignored. New exception Malformed_code_read
