@@ -1,4 +1,4 @@
-(* $Id: netencoding.ml,v 2.15 2004/07/31 16:00:54 stolpmann Exp $
+(* $Id: netencoding.ml,v 2.16 2005/04/14 12:44:40 stolpmann Exp $
  * ----------------------------------------------------------------------
  *
  *)
@@ -451,7 +451,7 @@ end
 
 module QuotedPrintable = struct
 
-  let encode_substring ?(eot = false) ?(line_length = ref 0) s ~pos ~len =
+  let encode_substring ?(crlf = true) ?(eot = false) ?(line_length = ref 0) s ~pos ~len =
     (* line_length:
      * - on input, the length of the line where the encoding starts
      * - on output, the length of the last written line
@@ -466,6 +466,8 @@ module QuotedPrintable = struct
     if pos + len > String.length s then
       invalid_arg "Netencoding.QuotedPrintable.encode";
 
+    let eol_len = if crlf then 2 else 1 in    (* length of eol *)
+
     (* Note: The [count] algorithm must strictly correspond to the
      * "for" loop below.
      *)
@@ -479,15 +481,15 @@ module QuotedPrintable = struct
 	match String.unsafe_get s (pos+i) with
 	    '\r' ->              (* CR is deleted *)
 	      count l n (i+1)
-	  | '\n' ->              (* LF is expanded to CR/LF *)
-	      count 0 (n+2) (i+1)
+	  | '\n' ->              (* LF may be expanded to CR/LF *)
+	      count 0 (n+eol_len) (i+1)
 	  | ('\000'..'\031'|'\127'..'\255'|
 	     '!'|'"'|'#'|'$'|'@'|'['|']'|'^'|'\''|'{'|'|'|'}'|'~'|'=') ->
 	      if l <= 69 then
 		count (l+3) (n+3) (i+1)
 	      else
 		(* Add soft line break after the encoded char: *)
-		count 0 (n+6) (i+1)
+		count 0 (n+4+eol_len) (i+1)
 	  | 'F' when l=0 ->
 	      (* Protect 'F' at the beginning of lines *)
 	      count (l+3) (n+3) (i+1)
@@ -503,11 +505,11 @@ module QuotedPrintable = struct
 		count (l+3) (n+3) (i+1)
 	      else
 		(* Add soft line after the encoded space: *)
-		count 0 (n+6) (i+1)
+		count 0 (n+4+eol_len) (i+1)
 	  | _ ->
 	      if l>71 then
 		(* Add soft line break after the char: *)
-		count 0 (n+4) (i+1)
+		count 0 (n+2+eol_len) (i+1)
 	      else
 		count (l+1) (n+1) (i+1)
       else
@@ -531,8 +533,12 @@ module QuotedPrintable = struct
 
     let add_soft_break() =
       t.[ !k ]   <- '=';
-      t.[ !k+1 ] <- '\r';
-      t.[ !k+2 ] <- '\n';
+      if crlf then (
+	t.[ !k+1 ] <- '\r';
+	t.[ !k+2 ] <- '\n';
+      )
+      else
+	t.[ !k+1 ] <- '\n';
     in
 
     (* In the following, the soft break criterion is [!l > 72]. Why?
@@ -548,9 +554,14 @@ module QuotedPrintable = struct
 	  '\r' ->   (* CR is deleted *)
 	    ()
 	| '\n' ->   (* LF is expanded to CR/LF *)
-	    t.[ !k ] <- '\r';
-	    t.[ !k+1 ] <- '\n';
-	    k := !k + 2;
+	    if crlf then (
+	      t.[ !k ] <- '\r';
+	      t.[ !k+1 ] <- '\n';
+	      k := !k + 2;
+	    ) else (
+	      t.[ !k ] <- '\n';
+	      k := !k + 1;
+	    );
 	    l := 0
 	| ('\000'..'\031'|'\127'..'\255'|
 	   '!'|'"'|'#'|'$'|'@'|'['|']'|'^'|'\''|'{'|'|'|'}'|'~'|'=') as c ->
@@ -560,7 +571,7 @@ module QuotedPrintable = struct
 	    if !l > 72 then (
 	      (* Add soft line break: *)
 	      add_soft_break();
-	      k := !k + 3;
+	      k := !k + 1 + eol_len;
 	      l := 0
 	    )
 	| 'F' when !l = 0 ->
@@ -576,7 +587,7 @@ module QuotedPrintable = struct
 	    l := !l + 3;
 	    if !l > 72 then (
 	      add_soft_break();
-	      k := !k + 3;
+	      k := !k + 1 + eol_len;
 	      l := 0;
 	    )
 	| c ->
@@ -585,7 +596,7 @@ module QuotedPrintable = struct
 	    incr l;
 	    if !l > 72 then (
 	      add_soft_break();
-	      k := !k + 3;
+	      k := !k + 1 + eol_len;
 	      l := 0;
 	    )
     done;
@@ -597,12 +608,12 @@ module QuotedPrintable = struct
     t ;;
 
 
-  let encode ?(pos=0) ?len s =
+  let encode ?crlf ?(pos=0) ?len s =
     let l = match len with None -> String.length s - pos | Some x -> x in 
-    encode_substring ~eot:true s ~pos ~len:l;;
+    encode_substring ?crlf ~eot:true s ~pos ~len:l;;
 
 
-  let encoding_pipe_conv line_length incoming incoming_eof outgoing =
+  let encoding_pipe_conv ?crlf line_length incoming incoming_eof outgoing =
     (* Problematic case: the incoming buffer ends with a space, but we are
      * not at EOF. It is possible that a LF immediately follows, and that
      * the space needs to be quoted.
@@ -616,15 +627,15 @@ module QuotedPrintable = struct
       else
 	(len, true)
     in
-    let s' = encode_substring ~eot ~line_length s ~pos:0 ~len:len' in
+    let s' = encode_substring ?crlf ~eot ~line_length s ~pos:0 ~len:len' in
     Netbuffer.add_string outgoing s';
     Netbuffer.delete incoming 0 len'
   ;;
     
 
-  class encoding_pipe () =
+  class encoding_pipe ?crlf () =
     let line_length = ref 0 in
-    Netchannels.pipe ~conv:(encoding_pipe_conv line_length) ()
+    Netchannels.pipe ~conv:(encoding_pipe_conv ?crlf line_length) ()
 
 
   let decode_substring s ~pos ~len =
@@ -1573,6 +1584,10 @@ end
  * History:
  * 
  * $Log: netencoding.ml,v $
+ * Revision 2.16  2005/04/14 12:44:40  stolpmann
+ * One can now select LF as line terminator for all message writing
+ * functions. Netsendmail.sendmail defaults to LF for Unix compatiblity.
+ *
  * Revision 2.15  2004/07/31 16:00:54  stolpmann
  * 	Bugfix QuotedPrintable: Space at end of string no longer causes
  * exception
