@@ -1,4 +1,4 @@
-(* $Id: netencoding.mli,v 2.8 2004/07/10 23:43:06 stolpmann Exp $
+(* $Id: netencoding.mli,v 2.9 2005/04/14 12:44:40 stolpmann Exp $
  * ----------------------------------------------------------------------
  *
  *)
@@ -106,13 +106,14 @@ module QuotedPrintable :
      * described in RFC 2045.
      *
      * This implementation assumes that the encoded string has a text MIME
-     * type. The character CR is removed from the input, and LF is
-     * replaced by CR/LF. Note that this implies that
-     * - the output uses CR/LF as line separator as MIME prescribes
+     * type. On input both CR/LF and LF are accepted as end-of-line (eol) terminators,
+     * but the output normalizes the eol delimiter as the [crlf] argument
+     * specifies. Note that this implies that
+     * - If [crlf], the output uses CR/LF as line separator as MIME prescribes
      * - the encoding is not invertible for binary data
      *)
 
-    val encode : ?pos:int -> ?len:int -> string -> string
+    val encode : ?crlf:bool -> ?pos:int -> ?len:int -> string -> string
 	(** Encodes the string and returns it.
 	 *
 	 * Since OcamlNet 0.98, soft line breaks are added to the output
@@ -129,6 +130,9 @@ module QuotedPrintable :
 	 * If [pos] and/or [len] are passed, only the substring starting at
 	 * [pos] (default: 0) with length [len] (default: rest of the string)
 	 * is encoded.
+	 *
+          * If [crlf] is set (the default), the output text uses CR/LF as
+	 * line separator. Otherwise only LF is used.
 	 *)
 
     val decode : ?pos:int -> ?len:int -> string -> string
@@ -141,7 +145,7 @@ module QuotedPrintable :
 	 * is decoded.
 	 *)
 
-    class encoding_pipe : unit -> Netchannels.pipe
+    class encoding_pipe : ?crlf:bool -> unit -> Netchannels.pipe
       (** This pipe encodes the data written into the pipe. *)
 
     class decoding_pipe : unit -> Netchannels.pipe
@@ -359,6 +363,10 @@ module Html :
  * History:
  * 
  * $Log: netencoding.mli,v $
+ * Revision 2.9  2005/04/14 12:44:40  stolpmann
+ * One can now select LF as line terminator for all message writing
+ * functions. Netsendmail.sendmail defaults to LF for Unix compatiblity.
+ *
  * Revision 2.8  2004/07/10 23:43:06  stolpmann
  * 	ocamldoc
  *
