@@ -1,4 +1,4 @@
-(* $Id: nethtml.ml,v 2.8 2004/07/08 23:04:25 stolpmann Exp $
+(* $Id: nethtml.ml,v 2.9 2005/01/22 10:58:26 stolpmann Exp $
  * ----------------------------------------------------------------------
  *
  *)
@@ -130,10 +130,12 @@ let html40_dtd =
     "dl",                 (`Block, `Elements ["dt"; "dd"]);
     "div",                (`Block, `Flow);
     "noscript",           (`Block, `Flow);
-    "blockquote",         (`Block, (`Block |. `Elements ["script"]));
+    "blockquote",         (`Block, (`Flow |. `Elements ["script"]));
+                          (* strict DTD has `Block here *)
     "form",               (`Block, `Sub_exclusions( ["form"],
-						    `Block |. 
+						    `Flow |. 
 						       `Elements ["script"]));
+                          (* strict DTD has `Block here *)
     "hr",                 (`Block, `Empty);
     "table",              (`Block, `Elements ["caption"; "col"; "colgroup";
 					      "thead"; "tfoot"; "tbody"; "tr"]);
@@ -145,6 +147,7 @@ let html40_dtd =
     "isindex",            (`Block, `Empty);
     (* ------------ OTHER ELEMENTS ----------*)
     "body",               (`None, (`Flow |. `Elements ["script"]));
+                          (* strict DTD has `Block here *)
     "area",               (`None, `Empty);
     "link",               (`None, `Empty);
     "param",              (`None, `Empty);
@@ -711,6 +714,10 @@ let write ?(dtd = html40_dtd) ch doc =
  * History:
  * 
  * $Log: nethtml.ml,v $
+ * Revision 2.9  2005/01/22 10:58:26  stolpmann
+ * Error: FORM, BLOCKQUOTE: Content is "flow" in the transitional DTD, not
+ * "block" as in the strict DTD.
+ *
  * Revision 2.8  2004/07/08 23:04:25  stolpmann
  * 	ocamldoc
  *
