@@ -1,4 +1,4 @@
-(* $Id: netmime.ml,v 1.9 2004/07/06 17:24:22 stolpmann Exp $
+(* $Id: netmime.ml,v 1.10 2005/04/14 12:44:40 stolpmann Exp $
  * ----------------------------------------------------------------------
  *
  *)
@@ -557,7 +557,7 @@ let decode_mime_body hdr =
 ;;
 
 
-let encode_mime_body hdr =
+let encode_mime_body ?(crlf = true) hdr =
   let encoding =
     try hdr # content_transfer_encoding()
     with Not_found -> "7bit"
@@ -569,11 +569,11 @@ let encode_mime_body hdr =
 	(fun s -> 
 	   new output_filter 
 	     (new Netencoding.Base64.encoding_pipe 
-	       ~linelength:76 ~crlf:true ()) s)
+	       ~linelength:76 ~crlf ()) s)
     | "quoted-printable" ->
 	(fun s ->
 	   new output_filter
-	     (new Netencoding.QuotedPrintable.encoding_pipe()) s)
+	     (new Netencoding.QuotedPrintable.encoding_pipe ~crlf ()) s)
     | _ ->
 	failwith "Netmime.encode_mime_body: Unknown Content-transfer-encoding"
 ;;
@@ -687,9 +687,11 @@ let rec augment_message (hdr,cbody) =
 
 
 let rec write_mime_message_int ?(wr_header = true) ?(wr_body = true) ?(nr = 0) 
-                               ?ret_boundary
+                               ?ret_boundary ?(crlf = true)
                                outch (hdr,cbody) =
 
+  let eol = if crlf then "\r\n" else "\n" in
+
   let mk_boundary parts =
     (* For performance reasons, gather random data only from the first
      * `Body
@@ -720,9 +722,9 @@ let rec write_mime_message_int ?(wr_header = true) ?(wr_body = true) ?(nr = 0)
       `Body body ->
 	(* Write the header as it is, and append the body *)
 	if wr_header then
-	  Mimestring.write_header outch hdr#fields;
+	  Mimestring.write_header ~eol ~soft_eol:eol outch hdr#fields;
 	if wr_body then begin
-	  let outch' = encode_mime_body hdr outch in
+	  let outch' = encode_mime_body ~crlf hdr outch in
 	  with_in_obj_channel
 	    (body # open_value_rd())
 	    (fun bodych -> outch' # output_channel bodych);
@@ -759,7 +761,7 @@ let rec write_mime_message_int ?(wr_header = true) ?(wr_body = true) ?(nr = 0)
 		  let b = mk_boundary parts in
 		  let ctype_field = 
 		    hdr # field "content-type" ^ 
-		    ";\r\n boundary=\"" ^ b ^ "\""  in
+		    ";" ^ eol ^ " boundary=\"" ^ b ^ "\""  in
 		  hdr' # update_field "content-type" ctype_field;
 		  b
 	  with
@@ -767,7 +769,7 @@ let rec write_mime_message_int ?(wr_header = true) ?(wr_body = true) ?(nr = 0)
 		(* Add the missing content-type header: *)
 		let b = mk_boundary parts in
 		let ctype_field = 
-		  "multipart/mixed;\r\n boundary=\"" ^ b ^ "\"" in
+		  "multipart/mixed;" ^ eol ^ " boundary=\"" ^ b ^ "\"" in
 		hdr' # update_field "content-type" ctype_field;
 		b
 	in
@@ -775,19 +777,19 @@ let rec write_mime_message_int ?(wr_header = true) ?(wr_body = true) ?(nr = 0)
 	hdr' # update_field "content-transfer-encoding" cte;
 	(* Write now the header fields *)
 	if wr_header then
-	  Mimestring.write_header outch hdr'#fields;
+	  Mimestring.write_header ~eol ~soft_eol:eol outch hdr'#fields;
 	(* Write the parts: *)
 	if wr_body then begin
-	  let boundary_string = "--" ^ boundary ^ "\r\n" in
+	  let boundary_string = "--" ^ boundary ^ eol in
 	  List.iter
 	    (fun part ->
 	       outch # output_string boundary_string;
 	       write_mime_message_int 
 	         ~wr_header:true ~wr_body:true ~nr:(nr + 1) outch part;
-	       outch # output_string "\r\n";
+	       outch # output_string eol;
 	    )
 	    parts;
-	  outch # output_string ("--" ^ boundary ^ "--\r\n");
+	  outch # output_string ("--" ^ boundary ^ "--" ^ eol);
 	end;
 	( match ret_boundary with
 	      None -> ()
@@ -796,9 +798,9 @@ let rec write_mime_message_int ?(wr_header = true) ?(wr_body = true) ?(nr = 0)
 ;;
 
 
-let write_mime_message ?wr_header ?wr_body ?nr ?ret_boundary ch msg =
+let write_mime_message ?wr_header ?wr_body ?nr ?ret_boundary ?crlf ch msg =
   write_mime_message_int 
-     ?wr_header ?wr_body ?nr ?ret_boundary
+     ?wr_header ?wr_body ?nr ?ret_boundary ?crlf
      ch (augment_message msg)
 ;;
 
@@ -807,6 +809,10 @@ let write_mime_message ?wr_header ?wr_body ?nr ?ret_boundary ch msg =
  * History:
  * 
  * $Log: netmime.ml,v $
+ * Revision 1.10  2005/04/14 12:44:40  stolpmann
+ * One can now select LF as line terminator for all message writing
+ * functions. Netsendmail.sendmail defaults to LF for Unix compatiblity.
+ *
  * Revision 1.9  2004/07/06 17:24:22  stolpmann
  * 	basic_mime_header respects the order of fields even when fields
  * with a certain name are not adjacent. Changed the impl of basic_mime_header
