@@ -1,4 +1,4 @@
-(* $Id: netsendmail.ml,v 1.3 2004/07/04 18:49:53 stolpmann Exp $
+(* $Id: netsendmail.ml,v 1.4 2005/04/14 12:44:40 stolpmann Exp $
  * ----------------------------------------------------------------------
  *
  *)
@@ -393,12 +393,12 @@ let compose
 ;;
 
 
-let sendmail ?(mailer = sendmail_program) message =
+let sendmail ?(mailer = sendmail_program) ?(crlf = false) message =
   let cmd = mailer ^ " -B8BITMIME -t -i" in
   with_out_obj_channel
     (new output_command cmd)
     (fun ch ->
-       write_mime_message ch message;
+       write_mime_message ~crlf ch message;
     )
 ;;
 
@@ -406,6 +406,10 @@ let sendmail ?(mailer = sendmail_program) message =
  * History:
  * 
  * $Log: netsendmail.ml,v $
+ * Revision 1.4  2005/04/14 12:44:40  stolpmann
+ * One can now select LF as line terminator for all message writing
+ * functions. Netsendmail.sendmail defaults to LF for Unix compatiblity.
+ *
  * Revision 1.3  2004/07/04 18:49:53  stolpmann
  * 	Revised interface: Mails can also be composed with
  * wrap_mail, wrap_parts, and wrap_attachment, in addition to the
