@@ -1,4 +1,4 @@
-(* $Id: netstring_top.ml,v 2.2 2002/01/12 18:35:46 stolpmann Exp $
+(* $Id: netstring_top.ml,v 2.4 2004/06/30 22:27:42 stolpmann Exp $
  * ----------------------------------------------------------------------
  *
  *)
@@ -16,13 +16,22 @@ exec "#install_printer Mimestring.print_s_param;;";;
 exec "#install_printer Neturl.print_url;;";;
 exec "#install_printer Netbuffer.print_buffer;;";;
 exec "#install_printer Netstream.print_in_obj_stream;;";;
-exec "#install_printer Netstream.Deprecated.print_stream;;";;
-exec "#install_printer Cgi.print_argument;;";;
+(* exec "#install_printer Cgi.print_argument;;";; *)
 
 (* ======================================================================
  * History:
  * 
  * $Log: netstring_top.ml,v $
+ * Revision 2.4  2004/06/30 22:27:42  stolpmann
+ * 	Update: Cgi is optional, -enable-compatcgi still possible.
+ *
+ * Revision 2.3  2004/05/30 21:27:14  stolpmann
+ * 	Supporting the standard for class-based I/O. This also means
+ * the semantics of [input] and [output] methods have changed (return
+ * value 0, End_of_file).
+ * 	Removed _rep_in, _rep_out.
+ * 	Removed Netstream.Deprecated.
+ *
  * Revision 2.2  2002/01/12 18:35:46  stolpmann
  * 	Updated the toploop printers.
  *
